<?php

class WPSFG_Instagram 
{

	private $config;
	private $cacheInstance;

	function __construct( $config )
	{
		$this->config = $config;
	}

	function drawWidget()
	{

		
		$username 		= trim( strtolower( $this->config['username'] ) );
		$columns	  	= isset($this->config['columns'])?trim( strtolower( $this->config['columns'] ) ):2;
		$image_size		= isset($this->config['image_size'])?trim( strtolower( $this->config['image_size'] ) ):'thumbnail';

		switch ( substr( $username, 0, 1 ) ) {
			case '#':
				$url              = 'https://www.instagram.com/explore/tags/' . str_replace( '#', '', $username );
				$transient_prefix = 'hashtag';
				$location_text    = esc_html__( 'Visit', 'wpsfg' );
				break;

			default:
				$url              = 'https://www.instagram.com/' . str_replace( '@', '', $username );
				$transient_prefix = 'user';
				$location_text    = esc_html__( 'Follow', 'wpsfg' );
				break;
		}


		$username = sanitize_user($username, false);
	    $username = preg_replace( '|[^#a-z0-9_.@]|i', '', $username );


		//Setting up cache request
		$this->cacheInstance = 'wpsfg-' . $transient_prefix . '-' .  str_replace(array('@', '#'), '', $username);

		//Setting up Output
		$output = '';

		//delete_transient( $this->cacheInstance );

		//Get from Instagram and Save it in Transient
		if ( false === ( $instagram = get_transient( $this->cacheInstance ) ) ) {
			$instagram = $this->getPageRequest( $url );


			// do not set an empty transient - should help catch private or empty accounts.
			if ( !is_wp_error( $instagram ) && !empty( $instagram ) ) {
				set_transient( $this->cacheInstance, $instagram, apply_filters( 'wpsfg_cache_time', HOUR_IN_SECONDS * 2 ) );
			}
		}


		//If contain a WP Error Instance
		if ( is_wp_error( $instagram ) ) {
			$output .= '
			<div class="wpsfg-widget wpsfg-'.$transient_prefix.'">
				<div class="wpsfg-header">
					<div class="wpsfg-user-info" title="' . esc_html__( 'Unknown Account', 'wpsfg' ) . '">'.$username.'</div>
			';
			//If is User?
			if ( $transient_prefix == "user" ){
				$output .= '
					<div class="wpsfg-info">
						<div class="wpsfg-avatar"><img src="' . plugins_url('assets/images/wpsfg-instagram.jpg', WPSFG_FILE) . '" alt=""></div>
						<div class="wpsfg-stats">
							<div class="wpsfg-stat">
								<strong>0</strong>
								<span>' . esc_html__( 'posts', 'wpsfg' ) . '</span>
							</div>
							<div class="wpsfg-stat">
								<strong>0</strong>
								<span>' . esc_html__( 'followers', 'wpsfg' ) . '</span>
							</div>
							<div class="wpsfg-stat">
								<strong>0</strong>
								<span>' . esc_html__( 'following', 'wpsfg' ) . '</span>
							</div>
						</div>
					</div>';
			}

			$output .= '
				</div>
				<div class="wpsfg-gallery">
					<div class="wpsfg-info-box">' . wp_kses_post( $instagram->get_error_message() ) . '</div>
				</div>
			</div>
			';
        }else{

			$instagram_data = $instagram;
			
			$output .= '
			<div class="wpsfg-widget wpsfg-'.$transient_prefix.'">
				<div class="wpsfg-header">
					<div class="wpsfg-user-info" title="' . (isset($instagram_data['user']['full_name'])?$instagram_data['user']['full_name']:$username) . '">'.$username.'<a href="' . $url . '" class="wpsfg-button" target="_blank">' . $location_text . '</a></div>
			';

			//If is User?
			if ( $transient_prefix == "user" ){
				$output .= '
					<div class="wpsfg-info">
						<div class="wpsfg-avatar"><a href="' . $url . '" target="_blank"><img src="' . $instagram_data['user']['profile_pic_url'] . '" alt="' . $instagram_data['user']['username'] . '"></a></div>
						<div class="wpsfg-stats">
							' . (isset($instagram_data['user']['total_posts'])?'
							<div class="wpsfg-stat">
								<strong>' . $this->number_shorten($instagram_data['user']['total_posts']) . '</strong>
								<span>' . esc_html__( 'posts', 'wpsfg' ) . '</span>
							</div>
							':'') . '
							' . (isset($instagram_data['user']['edge_followed_by'])?'
							<div class="wpsfg-stat">
								<strong>' . $this->number_shorten($instagram_data['user']['edge_followed_by']) . '</strong>
								<span>' . esc_html__( 'followers', 'wpsfg' ) . '</span>
							</div>
							':'') . '
							' . (isset($instagram_data['user']['edge_follow'])?'
							<div class="wpsfg-stat">
								<strong>' . $this->number_shorten($instagram_data['user']['edge_follow']) . '</strong>
								<span>' . esc_html__( 'following', 'wpsfg' ) . '</span>
							</div>
							':'') . '
						</div>
					</div>
				';
			}

			$output .= '
				</div>
				<div class="wpsfg-gallery">
				';

			//If feed is Private
			if (isset($instagram_data['user']['is_private']) && $instagram_data['user']['is_private'] == "1"){
				$output .= '<div class="wpsfg-info-box">' . sprintf(esc_html__( "%s's Instagram Account is Private.", 'wpsfg' ), '<strong>' . $username . '</strong>') . '</div>';
			}

			//If have media?
			if ( isset($instagram_data['media']) && !empty($instagram_data['media']) ){

				$media_divs = '';
				foreach ($instagram_data['media'] as $image){
					if ( $this->showInstaFeed( $image['type'] ) )
					{
						$media_divs .= '<div class="wpsfg-media">';
						$media_divs .= '<a class="wpsfg-' . $image['type'] . '" target="_blank" href="' . $image['link'] . '"><img src="' . $image[ $image_size ] . '" alt=""></a>';
						$media_divs .= '</div>';
					}
				}
				if (!empty($media_divs)){
					$output .= '<div class="wpsfg-gallery-body wpsfg-' . $columns . '-column">';
					$output .= $media_divs;
					$output .= '</div>';
				}else{
					$output .= '<div class="wpsfg-info-box">' . sprintf(esc_html__( 'Sorry, no %s found for the feed.', 'wpsfg' ), $this->config['type']) . '</div>';
				}
			}

			$output .= '
				</div>
			</div>
			';

		}
		return $output;
	}

	private function showInstaFeed($type)
	{
		if (!isset($this->config['type']) || (isset($this->config['type']) && !in_array($this->config['type'], array('image', 'video'))) )
		{
			return true;
		}

		if ( isset($this->config['type']) && $this->config['type']=="image" && $type=='image'){
			return true;
		}

		if ( isset($this->config['type']) && $this->config['type']=="video" && $type=='video'){
			return true;
		}

		return false;
	}

	private function getPageRequest( $url )
	{


		$pageRequest = $this->__get_requests( $url );

		if (!preg_match('#window\._sharedData\s?=\s?(.*);<\/script>#', $pageRequest, $page_data_matches)) {
            return new WP_Error( 'bad_json', esc_html__( 'Instagram has returned invalid data.', 'wpsfg' ) );
        }else{

        	$page_data = json_decode($page_data_matches[1], true);

        	if (empty($page_data) || empty($page_data['entry_data'])) {
        		return new WP_Error( 'bad_json', esc_html__( 'Instagram has returned empty page data.', 'wpsfg' ) );
	        }

	        $totalPosts = 0;
	        $user = array();
	        if ( isset( $page_data['entry_data']['ProfilePage'][0]['graphql']['user']['edge_owner_to_timeline_media']['edges'] ) ) {
				$images 	= $page_data['entry_data']['ProfilePage'][0]['graphql']['user']['edge_owner_to_timeline_media']['edges'];
				$userInfo 	= $page_data['entry_data']['ProfilePage'][0]['graphql']['user'];
				$totalPosts = $page_data['entry_data']['ProfilePage'][0]['graphql']['user']['edge_owner_to_timeline_media']['count'];

				foreach ($this->captureFields() as $field){
					if ( isset($userInfo[ $field ]) ){
						if ( in_array($field, array( 'edge_followed_by', 'edge_follow' )) ){
							$user[ $field ] = $userInfo[ $field ]['count'];
						}else{
							$user[ $field ] = $userInfo[ $field ];
						}
					}
				}

				$user['total_posts'] = $totalPosts;

			} elseif ( isset( $page_data['entry_data']['TagPage'][0]['graphql']['hashtag']['edge_hashtag_to_media']['edges'] ) ) {
				if ( 
					isset( $page_data['entry_data']['TagPage'][0]['graphql']['hashtag']['edge_hashtag_to_media']['edges'] ) && 
					empty( $page_data['entry_data']['TagPage'][0]['graphql']['hashtag']['edge_hashtag_to_media']['edges'] )
				){
					$images = $page_data['entry_data']['TagPage'][0]['graphql']['hashtag']['edge_hashtag_to_top_posts']['edges'];
				}else{
					$images = $page_data['entry_data']['TagPage'][0]['graphql']['hashtag']['edge_hashtag_to_media']['edges'];
				}
			} else {
				return new WP_Error( 'bad_json_2', esc_html__( 'Instagram has returned invalid data.', 'wpsfg' ) );
			}

			if ( ! is_array( $images ) ) {
				return new WP_Error( 'no_media', esc_html__( 'Instagram has returned no media.', 'wpsfg' ) );
			}

			$body = $media = array();

			foreach ( $images as $image ) {
				if ( true === $image['node']['is_video'] ) {
					$type = 'video';
				} else {
					$type = 'image';
				}

				$caption = __( 'Instagram Image', 'wpsfg' );
				if ( ! empty( $image['node']['edge_media_to_caption']['edges'][0]['node']['text'] ) ) {
					$caption = wp_kses( $image['node']['edge_media_to_caption']['edges'][0]['node']['text'], array() );
				}

				$media[] = array(
					'description' => $caption,
					'link'        => trailingslashit( '//instagram.com/p/' . $image['node']['shortcode'] ),
					'time'        => $image['node']['taken_at_timestamp'],
					'comments'    => $image['node']['edge_media_to_comment']['count'],
					'likes'       => $image['node']['edge_liked_by']['count'],
					'thumbnail'   => preg_replace( '/^https?\:/i', '', $image['node']['thumbnail_resources'][0]['src'] ),
					'small'       => preg_replace( '/^https?\:/i', '', $image['node']['thumbnail_resources'][2]['src'] ),
					'large'       => preg_replace( '/^https?\:/i', '', $image['node']['thumbnail_resources'][4]['src'] ),
					'original'    => preg_replace( '/^https?\:/i', '', $image['node']['display_url'] ),
					'type'        => $type,
				);
			} // End foreach().

			$body['user']  = $user;
			$body['media'] = $media;

        }

		return $body;
	}

	private static function captureFields(){
		return array('id', 'username', 'full_name', 'biography', 'edge_followed_by', 'edge_follow', 'highlight_reel_count', 'has_channel', 'is_business_account', 'is_joined_recently', 'is_verified', 'is_private', 'profile_pic_url', 'profile_pic_url_id');
	}

	private function __get_requests( $url ){

		$cookies      = array();
		foreach ( array(
            'ig_or' => 'landscape-primary',
            'ig_pr' => '1',
            'ig_vh' => 1080,
            'ig_vw' => 1920,
            'ds_user_id' => 25025320
        ) as $name => $value ) {
		    $cookies[] = new WP_Http_Cookie( array( 'name' => $name, 'value' => $value ) );
		}

		$request = wp_remote_get( $url, array( 
			'timeout' => 120, 
			'httpversion' => '1.1',
            'User-Agent' => 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.87 Safari/537.36',
            'Origin' => 'https://www.instagram.com',
            'Referer' => 'https://www.instagram.com',
            'Connection' => 'close',
            //'cookies' => $cookies
		) );

		$body    = wp_remote_retrieve_body( $request );

		return $body;
	}

	// Shortens a number and attaches K, M, B, etc. accordingly
	static function number_shorten($number, $precision = 2, $divisors = null) {

	    if ($number < 900) {
	        // Default
	         $number_format = number_format($number);
        } else if ($number < 900000) {
	        // Thausand
	        $number_format = number_format($number / 1000, $precision). 'K';
        } else if ($number < 900000000) {
	        // Million
	        $number_format = number_format($number / 1000000, $precision). 'M';
        } else if ($number < 900000000000) {
	        // Billion
	        $number_format = number_format($number / 1000000000, $precision). 'B';
        } else {
	        // Trillion
	        $number_format = number_format($number / 1000000000000, $precision). 'T';
	    }
	    return $number_format;
	}

}

?>