<?php
/**
 * Class WPDocs_New_Widget
 */
class WPSFG_Instagram_Widget extends WP_Widget {
 
    /**
     * Constructs the new widget.
     *
     * @see WP_Widget::__construct()
     */
    function __construct() {

		$widget_ops  = array(
			'classname'                   => 'widget_wpsfg',
			'description'                 => '',
			'customize_selective_refresh' => true,
		);

        // Instantiate the parent object.
		parent::__construct( 'wpsfg', esc_html__('WP Social Feed Gallery', 'wpsfg'), $widget_ops );
    }
 
    /**
     * The widget's HTML output.
     *
     * @see WP_Widget::widget()
     *
     * @param array $args     Display arguments including before_title, after_title,
     *                        before_widget, and after_widget.
     * @param array $instance The settings for the particular instance of the widget.
     */
    function widget( $args, $instance ) {

    	echo $args['before_widget'];

    	//Getting Options
		$options = $GLOBALS['WPSFG']->getSettings();

		//Fallback Username
		$settings_username = $options['wpsfg-username'];

		//Widget Username
		$username = $instance['username'];
		if (empty($username))
		{
			$username = $settings_username;
		}
		$username = sanitize_user($username, false);
	    $username = preg_replace( '|[^#a-z0-9_.@]|i', '', $username );

		$instagram = new WPSFG_Instagram (array(
        	'username' 		=> $username,
        	'columns'  		=> $instance['columns'],
        	'image_size'  	=> $instance['image_size'],
        	'type'  		=> $instance['type'],
        ));
        echo $instagram->drawWidget();

		echo $args['after_widget'];
    }
 
    /**
     * The widget update handler.
     *
     * @see WP_Widget::update()
     *
     * @param array $new_instance The new instance of the widget.
     * @param array $old_instance The old instance of the widget.
     * @return array The updated instance of the widget.
     */
    function update( $new_instance, $old_instance ) {

    	$new_instance = wp_parse_args(
			$new_instance,
			array(
				'username' => '',
				'image_size'  => 'thumbnail',
				'columns' => 2,
				'type'	  => 'both',
			)
		);

		$instance = $old_instance;

		if ( !empty($new_instance['username']) ) {
			$username = sanitize_user($new_instance['username'], false);
		    $username = preg_replace( '|[^#a-z0-9_.@]|i', '', $username );
			$instance['username'] = $username;
		} else {
			//Getting Options
			$instance['username'] = '';
		}

		$instance['image_size'] = $new_instance['image_size'];
		if ( empty( $instance['image_size'] ) ) {
			$instance['image_size'] = 'thumbnail';
		}

		$instance['columns'] = $new_instance['columns'];
		if ( empty( $instance['columns'] ) ) {
			$instance['columns'] = 2;
		}

		$instance['type'] = $new_instance['type'];
		if ( empty( $instance['type'] ) ) {
			$instance['type'] = 'both';
		}

		return $instance;
    }
 
    /**
     * Output the admin widget options form HTML.
     *
     * @param array $instance The current widget settings.
     * @return string The HTML markup for the form.
     */
    function form( $instance ) {
    	$instance = wp_parse_args(
			(array) $instance,
			array(
				'username' => '',
				'image_size'  => 'thumbnail',
				'columns' => 2,
				'type' => 'both',
			)
		);
		?>
		<p>
			<label for="<?php echo $this->get_field_id( 'username' ); ?>"><?php _e( 'Username or Hashtag:', 'wpsfg' ); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id( 'username' ); ?>" name="<?php echo $this->get_field_name( 'username' ); ?>" type="text" value="<?php echo esc_attr( $instance['username'] ); ?>"/>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'image_size' ); ?>"><?php _e( 'Image Size:', 'wpsfg' ); ?></label>
			<select class="widefat" id="<?php echo $this->get_field_id( 'image_size' ); ?>" name="<?php echo $this->get_field_name( 'image_size' ); ?>">
				<option value="thumbnail"		<?php selected(esc_attr( $instance['image_size'] ), 'thumbnail')?>><?php echo esc_html__( 'Thumbnail' , 'wpsfg' ) ?></option>
				<option value="small"			<?php selected(esc_attr( $instance['image_size'] ), 'small')?>><?php echo esc_html__( 'Small' , 'wpsfg' ) ?></option>
				<option value="large"			<?php selected(esc_attr( $instance['image_size'] ), 'large')?>><?php echo esc_html__( 'Large' , 'wpsfg' ) ?></option>
				<option value="original"		<?php selected(esc_attr( $instance['image_size'] ), 'original')?>><?php echo esc_html__( 'Original' , 'wpsfg' ) ?></option>
			</select>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'columns' ); ?>"><?php _e( 'Columns:', 'wpsfg' ); ?></label>
			<select class="widefat" id="<?php echo $this->get_field_id( 'columns' ); ?>" name="<?php echo $this->get_field_name( 'columns' ); ?>">
				<?php for($i=1;$i<=10;$i++){?>
				<option value="<?php echo $i;?>" <?php selected(esc_attr( $instance['columns'] ), $i)?>><?php echo sprintf(esc_html__( '%s Columns' , 'wpsfg' ), $i); ?></option>
				<?php }?>
			</select>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'type' ); ?>"><?php _e( 'Show in Feed:', 'wpsfg' ); ?></label>
			<select class="widefat" id="<?php echo $this->get_field_id( 'type' ); ?>" name="<?php echo $this->get_field_name( 'type' ); ?>">
				<option value="both"  <?php selected(esc_attr( $instance['type'] ), 'both')?>><?php echo esc_html__( 'Both Images &amp; Videos'  , 'wpsfg' ) ?></option>
				<option value="image" <?php selected(esc_attr( $instance['type'] ), 'image')?>><?php echo esc_html__( 'Images Only' , 'wpsfg' ) ?></option>
				<option value="video"  <?php selected(esc_attr( $instance['type'] ), 'video')?>><?php echo esc_html__( 'Videos Only'  , 'wpsfg' ) ?></option>
			</select>
		</p>
		<?php
    }
}
 
/**
 * Register the new widget.
 *
 * @see 'widgets_init'
 */
add_action( 'widgets_init', 'wpsfg_register_widgets');
function wpsfg_register_widgets() {
	register_widget( 'WPSFG_Instagram_Widget' );
}
?>