<?php
/*
Plugin Name: WP Social Feed Gallery
Plugin URI: https://github.com/wddportfolio/wp-social-feed-gallery
Description: WP Social Feed Gallery is a simple WordPress plugin that allow you to display your Instagram feed pictures in your website.
Version: 0.1.0
Author: WDDPortfolio
Author URI: https://www.wddportfolio.com/
Text Domain: wpsfg
Domain Path: /languages/
License: GPLv2 or later
License URI: http://www.gnu.org/licenses/gpl-2.0.html

Copyright © 2019 WDDPortfolio and Muhammad Furqan Abid

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

*/

class WPSFG {

  	private static $instance = null;
  	private $menuIcon;
    private $parentMenuId;
    private $aboutMenuId;

    public $pluginName;
    public $pluginSlug;
    public $pluginVersion = "0.1.0";

    public static $optionKey = "wpsfg-options";

  	function __construct()
  	{
  		// Defining Constants
  		self::constants();

  		// Initializing Actions
  		add_action( 'init', array($this, 'init') );

  		// load language files.
		load_plugin_textdomain( 'wpsfg', false, dirname( WPSFG_BASE ) . '/languages/' );
  	}

  	static function constants()
  	{
  		// define some constants.
		define( 'WPSFG_JS_URL', plugins_url( '/assets/js', __FILE__ ) );
		define( 'WPSFG_CSS_URL', plugins_url( '/assets/css', __FILE__ ) );
		define( 'WPSFG_IMAGES_URL', plugins_url( '/assets/images', __FILE__ ) );
		define( 'WPSFG_PATH', dirname( __FILE__ ) );
		define( 'WPSFG_BASE', plugin_basename( __FILE__ ) );
		define( 'WPSFG_FILE', __FILE__ );

		require_once ( WPSFG_PATH . '/includes/WelcomePage.php' );
		require_once ( WPSFG_PATH . '/includes/ConfigurationPage.php' );
		require_once ( WPSFG_PATH . '/includes/class-widget.php' );
		require_once ( WPSFG_PATH . '/includes/class-instagram.php' );
  	}

  	static function init()
  	{

		//
		$this->pluginName 		= esc_html__('WP Social Feed Gallery', 'wpsfg');
		$this->pluginSlug 		= 'wp-social-feed-gallery';
		$this->menuIcon 		= plugins_url('assets/images/wpsfg-icon.png', WPSFG_FILE);
		$this->pluginShortSlug  = 'wpsfg';
		$this->pluginWelcome	= new WPSFG_WelcomePage();
		$this->pluginConfigPage	= new WPSFG_ConfigurationPage();

		//Saving Options
		$this->saveSettings ();

		//Getting Options
		$options = $this->getSettings();

  		//Add Menu
  		add_action('admin_menu', array($this, 'addMenuPage'));

  		//Enqueue Styles & Scripts
  		add_action( 'wp_enqueue_scripts', array($this, 'front_style_scripts') );
  	}

  	static function addMenuPage()
  	{
  		$parentPage = array($this->pluginConfigPage, 'getPage');
  		$this->parentMenuId = add_menu_page( $this->pluginName, $this->pluginName, 'manage_options', $this->pluginSlug, $parentPage, $this->menuIcon);
  							  add_submenu_page( $this->pluginSlug, esc_html__('Configuration', 'wpsfg'), esc_html__('Configuration', 'wpsfg'), 'manage_options', $this->pluginSlug, $parentPage, $this->menuIcon);
  		$this->aboutMenuId  = add_submenu_page( $this->pluginSlug, esc_html__('About', 'wpsfg'), esc_html__('About', 'wpsfg'), 'manage_options', $this->pluginShortSlug.'-about', array($this->pluginWelcome, 'aboutPage'), 11 );

  		add_action( "load-{$this->parentMenuId}", array( &$this, '_load_page' ) );
  		add_action( "load-{$this->aboutMenuId}" , array( &$this, '_load_page' ) );
  	}

  	static function _load_page(){

  		// Add admin body class for this page
  		add_filter( 'admin_body_class', function($classes){
  			$classes .= ' wpsfg-page';
  			return $classes;
  		}, 10 );

  		// Do admin head action for this page
        // add_action( 'admin_head', array( &$this, 'admin_head' ) );

  		// Add required script and styles for this page
        add_action('admin_enqueue_scripts', array($this, 'enqueueAssets'));

        // Add Ajax
        add_action( 'wp_ajax_wpsfg-save'		, array( &$this->pluginConfigPage, 'updateSettings'), 10 );
        add_action( "wp_ajax_nopriv_wpsfg-save" , array( &$this->pluginConfigPage, 'updateSettings'), 10 );

        // Do admin footer text hook
        add_filter( 'admin_footer_text', array( &$this, 'admin_footer_text' ) );
		
  	}

  	static function front_style_scripts()
  	{

    	//Getting Options
		$options = self::getSettings();

		//Load Stylesheet at Frontend?
		if ($options['wpsfg-load-css'])
		{
  			wp_enqueue_style( 'wpsfg-style', plugins_url('assets/css/wpsfg-instagram-widget.css', WPSFG_FILE), array(), $this->pluginVersion );
  		}
  	}

  	static function enqueueAssets ( $hook ){
  		if ($hook && ($hook == $this->parentMenuId || $hook == $this->aboutMenuId)) {
  			wp_enqueue_style ($this->pluginShortSlug . '-admin', plugins_url('assets/wpsfg-admin.css', WPSFG_FILE), array(), $this->pluginVersion);
  		}
  	}

  	static function updateSettings ()
  	{
  		if (!wp_verify_nonce($_POST['wpsfg-nonce'], $this->pluginSlug)) {
            exit;
        }

        $this->saveSettings(false, false);

        $result = array();

        exit(json_encode($result));
  	}

  	static function admin_footer_text($text) 
  	{
        return '';
    }

    static function getSettings()
    {
    	//delete_option( self::$optionKey );
    	$options = get_option( self::$optionKey );
    	if ( empty( $options ) )
    	{
    		$options = array(
				'wpsfg-username'				 => 'instagram',
				'wpsfg-load-css'				 => true,
			);
    	}
    	return $options;
    }

    static function saveSettings($checkNonce=true, $redirect = true)
    {
    	if ( isset($_POST['action']) && $_POST['action']=="wpsfg-save" )
    	{
	    	if ($checkNonce && isset($_POST['wpsfg-nonce']) && !wp_verify_nonce($_POST['wpsfg-nonce'], $this->pluginSlug)) 
	    	{
	            exit;
	        }

	        $username = sanitize_user($_POST['wpsfg-username'], false);
	        $username = preg_replace( '|[^#a-z0-9_.@]|i', '', $username );

	        $optSave = array(
				'wpsfg-username'				 => $username,
				'wpsfg-load-css'				 => false
			);
			if ( isset($_POST['wpsfg-load-css']) && sanitize_title($_POST['wpsfg-load-css']) == "true" ){
				$optSave['wpsfg-load-css']	= true;
			}

			update_option( self::$optionKey, $optSave );

	        if ($redirect)
	        {
	        	wp_redirect( admin_url('admin.php?page=' . $GLOBALS['WPSFG']->pluginSlug) );
	        	exit;
	        }
        }

    }

	public static function getInstance()
	{
		if ( is_null( self::$instance ) )
			self::$instance = new WPSFG();

		return self::$instance;
	}
}

$GLOBALS['WPSFG'] = WPSFG::getInstance();

?>